@@ -29,10 +29,14 @@ from gtk import gdk, keysyms
 from kiwi.component import implements
 from kiwi.interfaces import IEasyCombo
 from kiwi.enums import ComboColumn, ComboMode
+from kiwi.log import Logger
 from kiwi.ui.entry import KiwiEntry
 from kiwi.ui.entrycompletion import KiwiEntryCompletion
 from kiwi.utils import gsignal, type_register
 
+log = Logger('kiwi.ui.combo')
+
+
 class _ComboEntryPopup(gtk.Window):
     gsignal('text-selected', str)
     def __init__(self, comboentry):
@@ -352,6 +356,7 @@ class ComboEntry(gtk.HBox):
 
     def __init__(self, entry=None):
         """
+        Create a new ComboEntry object.
         @param entry: a gtk.Entry subclass to use
         """
         gtk.HBox.__init__(self)
@@ -417,6 +422,8 @@ class ComboEntry(gtk.HBox):
 
     def _on_entry__scroll_event(self, entry, event):
         model = self.get_model()
+        if not len(model):
+            return
         treeiter = self._popup.get_selected_iter()
         # If nothing is selected, select the first one
         if not treeiter:
@@ -496,13 +503,15 @@ class ComboEntry(gtk.HBox):
 
     def set_text(self, text):
         """
+        Sets the text.
         @param text:
         """
         self.entry.set_text(text)
 
     def get_text(self):
         """
-        @returns: current text
+        Gets the current text.
+        @returns: the text.
         """
         return self.entry.get_text()
 
@@ -523,13 +532,15 @@ class ComboEntry(gtk.HBox):
 
     def get_model(self):
         """
-        @returns: our model
+        Gets our model.
+        @returns: model
         @rtype: gtk.TreeModel
         """
         return self._model
 
     def set_active_iter(self, iter):
         """
+        Set the iter selected.
         @param iter: iter to select
         @type iter: gtk.TreeIter
         """
@@ -540,7 +551,8 @@ class ComboEntry(gtk.HBox):
 
     def get_active_iter(self):
         """
-        @returns: the selected iter
+        Gets the selected iter.
+        @returns: iter selected.
         @rtype: gtk.TreeIter
         """
         return self._popup.get_selected_iter()
@@ -619,7 +631,12 @@ class ComboEntry(gtk.HBox):
         """
         See L{kiwi.interfaces.IEasyCombo.select}
         """
-        treeiter = self.entry.get_iter_from_obj(obj)
+        try:
+            treeiter = self.entry.get_iter_from_obj(obj)
+        except KeyError:
+            log.warn("%s does not contain a %r object" % (
+                self.__class__.__name__, obj))
+            return
         self.set_active_iter(treeiter)
 
     def append_item(self, label, data=None):
