@@ -24,14 +24,18 @@
 
 """Gazpacho integration: loader and extensions"""
 
-import datetime
-from decimal import Decimal
 import gettext
-import os
+import warnings
 
 import gobject
 import gtk
-from gazpacho.editor import PropertyCustomEditor
+
+try:
+    from gazpacho.propertyeditor import PropertyCustomEditor
+    PropertyCustomEditor # pyflakes
+except ImportError:
+    from gazpacho.editor import PropertyCustomEditor
+
 from gazpacho.loader.loader import ObjectBuilder
 from gazpacho.loader.custom import Adapter, ComboBoxAdapter, \
      PythonWidgetAdapter, adapter_registry
@@ -39,25 +43,30 @@ from gazpacho.properties import prop_registry, CustomProperty, StringType
 from gazpacho.widgets.base.base import ContainerAdaptor
 from gazpacho.widgets.base.box import BoxAdaptor
 
-from kiwi import _warn
-from kiwi.datatypes import currency
+from kiwi.datatypes import converter
 from kiwi.environ import environ
+from kiwi.log import Logger
+from kiwi.python import disabledeprecationcall
 from kiwi.ui.hyperlink import HyperLink
-from kiwi.ui.objectlist import Column, ObjectList
-from kiwi.ui.widgets.checkbutton import CheckButton
+from kiwi.ui.objectlist import Column, ObjectList, ObjectTree
+from kiwi.ui.widgets.button import ProxyButton
+from kiwi.ui.widgets.checkbutton import ProxyCheckButton
 from kiwi.ui.widgets.combo import ProxyComboEntry, ProxyComboBox, \
      ProxyComboBoxEntry
-from kiwi.ui.widgets.entry import ProxyDateEntry, Entry
-from kiwi.ui.widgets.label import Label
-from kiwi.ui.widgets.radiobutton import RadioButton
-from kiwi.ui.widgets.spinbutton import SpinButton
-from kiwi.ui.widgets.textview import TextView
+from kiwi.ui.widgets.entry import ProxyDateEntry, ProxyEntry
+from kiwi.ui.widgets.label import ProxyLabel
+from kiwi.ui.widgets.radiobutton import ProxyRadioButton
+from kiwi.ui.widgets.scale import ProxyHScale, ProxyVScale
+from kiwi.ui.widgets.spinbutton import ProxySpinButton
+from kiwi.ui.widgets.textview import ProxyTextView
 
 # Backwards compatibility + pyflakes
 from kiwi.ui.widgets.combobox import ComboBox, ComboBoxEntry
 from kiwi.ui.widgets.list import List
 HyperLink
-_ = gettext.gettext
+_ = lambda m: gettext.dgettext('kiwi', m)
+
+log = Logger('gazpacholoader')
 
 class Builder(ObjectBuilder):
     def find_resource(self, filename):
@@ -67,25 +76,21 @@ class GazpachoWidgetTree:
     """Example class of GladeAdaptor that uses Gazpacho loader to load the
     glade files
     """
-    def __init__(self, view, gladefile, widgets, gladename=None, domain=None):
-
-        if not gladefile:
-            raise ValueError("A gladefile wasn't provided.")
-        elif not isinstance(gladefile, basestring):
-            raise TypeError(
-                  "gladefile should be a string, found %s" % type(gladefile))
-        filename = os.path.splitext(os.path.basename(gladefile))[0]
-
+    def __init__(self, view, gladefile, domain=None):
         self._view = view
-        self._gladefile = environ.find_resource("glade", filename + ".glade")
-        self._widgets =  (widgets or view.widgets or [])[:]
-        self.gladename = gladename or filename
-        self._tree = Builder(self._gladefile, domain=domain)
-        if not self._widgets:
-            self._widgets = [w.get_data("gazpacho::object-id")
-                             for w in self._tree.get_widgets()]
+        self._gladefile = gladefile
+        self._showwarning = warnings.showwarning
+        warnings.showwarning = self._on_load_warning
+        self._tree = Builder(gladefile, domain=domain)
+        warnings.showwarning = self._showwarning
+        self._widgets = [w.get_data("gazpacho::object-id")
+                         for w in self._tree.get_widgets()]
         self._attach_widgets()
 
+    def _on_load_warning(self, warning, category, file, line):
+        self._showwarning('while loading glade file: %s' % warning,
+                          category, self._gladefile, '???')
+
     def _attach_widgets(self):
         # Attach widgets in the widgetlist to the view specified, so
         # widgets = [label1, button1] -> view.label1, view.button1
@@ -94,7 +99,7 @@ class GazpachoWidgetTree:
             if widget is not None:
                 setattr(self._view, w, widget)
             else:
-                _warn("Widget %s was not found in glade widget tree." % w)
+                log.warn("Widget %s was not found in glade widget tree." % w)
 
     def get_widget(self, name):
         """Retrieves the named widget from the View (or glade tree)"""
@@ -151,29 +156,34 @@ class ObjectListAdapter(PythonWidgetAdapter):
                                                         properties)
 adapter_registry.register_adapter(ObjectListAdapter)
 
+class ObjectTreeAdapter(PythonWidgetAdapter):
+    object_type = ObjectTree
+adapter_registry.register_adapter(ObjectTreeAdapter)
+
 # Framework widgets
 
 class DataTypeAdaptor(PropertyCustomEditor):
+    widget_type = None
+    default = str
+
     def __init__(self):
         super(DataTypeAdaptor, self).__init__()
+        self._model = None
         self._input = self.create_editor()
 
     def get_editor_widget(self):
         return self._input
 
-    def get_data_types(self):
-        """
-        Subclasses should override this.
-        Expected to return a list of 2 sized tuples with
-        name of type and type, to be used in a combo box.
-        """
-        raise NotImplementedError
+    def _get_converters(self):
+        if not self.widget_type:
+            AssertionError("%r should define a widget_type" % self)
+
+        allowed = self.widget_type.allowed_data_types
+        return converter.get_converters(allowed)
 
     def create_editor(self):
-        model = gtk.ListStore(str, object)
-        for datatype in self.get_data_types():
-            model.append(datatype)
-        combo = gtk.ComboBox(model)
+        self._model = gtk.ListStore(str, object)
+        combo = gtk.ComboBox(self._model)
         renderer = gtk.CellRendererText()
         combo.pack_start(renderer)
         combo.add_attribute(renderer, 'text', 0)
@@ -183,6 +193,11 @@ class DataTypeAdaptor(PropertyCustomEditor):
 
     def update(self, context, kiwiwidget, proxy):
         combo = self._input
+        model = self._model
+        model.clear()
+        for converter in self._get_converters():
+            model.append((converter.name, converter.type))
+
         connection_id = combo.get_data('connection-id')
         if (connection_id != -1):
             combo.disconnect(connection_id)
@@ -191,6 +206,8 @@ class DataTypeAdaptor(PropertyCustomEditor):
                                       proxy, model)
         combo.set_data('connection-id', connection_id)
         value = kiwiwidget.get_property('data-type')
+        if not value:
+            value = self.default
         for row in model:
             if row[1] == value:
                 combo.set_active_iter(row.iter)
@@ -201,66 +218,44 @@ class DataTypeAdaptor(PropertyCustomEditor):
         proxy.set_value(model[active_iter][1])
 
 class SpinBtnDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Currency'), currency)
-            ]
+    widget_type = ProxySpinButton
+    default = float
+
+class HScaleDataType(DataTypeAdaptor):
+    widget_type = ProxyHScale
+    default = float
+
+class VScaleDataType(DataTypeAdaptor):
+    widget_type = ProxyVScale
+    default = float
 
 class EntryDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('String'), str),
-            (_('Unicode'), unicode),
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Currency'), currency),
-            (_('Date'), datetime.date),
-            (_('Datetime'), datetime.datetime),
-            (_('Time'), datetime.time),
-            (_('Object'), object)
-            ]
+    widget_type = ProxyEntry
+    default = str
 
 class TextViewDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('String'), str),
-            (_('Unicode'), unicode),
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Date'), datetime.date),
-            ]
+    widget_type = ProxyTextView
+    default = str
 
 class ComboBoxDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('String'), str),
-            (_('Unicode'), unicode),
-            (_('Boolean'), bool),
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Object'), object)
-            ]
+    widget_type = ProxyComboBox
+    default = object
+
+class ComboBoxEntryDataType(DataTypeAdaptor):
+    widget_type = ProxyComboBoxEntry
+    default = object
+
+class ComboEntryDataType(DataTypeAdaptor):
+    widget_type = ProxyComboEntry
+    default = object
 
 class LabelDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('String'), str),
-            (_('Unicode'), unicode),
-            (_('Boolean'), bool),
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Date'), datetime.date),
-            (_('Datetime'), datetime.datetime),
-            (_('Time'), datetime.time),
-            (_('Currency'), currency)
-            ]
+    widget_type = ProxyLabel
+    default = str
+
+class ButtonDataType(DataTypeAdaptor):
+    widget_type = ProxyButton
+    default = str
 
 class DataType(CustomProperty, StringType):
     translatable = False
@@ -283,6 +278,19 @@ class DateOnlyDataType(CustomProperty, StringType):
 
 class ModelProperty(CustomProperty, StringType):
     translatable = False
+    has_custom_default = True
+
+    def __init__(self, gadget):
+        super(ModelProperty, self).__init__(gadget)
+        gadget.widget.connect('notify::name', self._on_widget__notify)
+
+    def _on_widget__notify(self, widget, pspec):
+        self.set(widget.get_name())
+        self.notify()
+
+    def default(self):
+        return self.gadget.widget.get_name()
+    default = property(default)
 
 class DataValueProperty(CustomProperty, StringType):
     translatable = False
@@ -302,37 +310,40 @@ class KiwiComboBoxAdapter(ComboBoxAdapter):
         else:
             raise AssertionError("Unknown ComboBox GType: %r" % gtype)
 
-        obj = object_type()
+        obj = disabledeprecationcall(object_type)
         obj.set_name(name)
         return obj
 adapter_registry.register_adapter(KiwiComboBoxAdapter)
 
 def register_widgets():
     for gobj, editor, data_type in [
-        (Entry, EntryDataType, DataType),
+        (ProxyEntry, EntryDataType, DataType),
         (ProxyDateEntry, None, DateOnlyDataType),
-        (CheckButton, None, BoolOnlyDataType),
-        (Label, LabelDataType, DataType),
+        (ProxyButton, ButtonDataType, DataType),
+        (ProxyCheckButton, None, BoolOnlyDataType),
+        (ProxyLabel, LabelDataType, DataType),
         (ProxyComboBox, ComboBoxDataType, DataType),
-        (ProxyComboBoxEntry, ComboBoxDataType, DataType),
-        (ProxyComboEntry, ComboBoxDataType, DataType),
-        (SpinButton, SpinBtnDataType, DataType),
-        (RadioButton, None, BoolOnlyDataType),
-        (TextView, TextViewDataType, DataType)
+        (ProxyComboBoxEntry, ComboBoxEntryDataType, DataType),
+        (ProxyComboEntry, ComboEntryDataType, DataType),
+        (ProxySpinButton, SpinBtnDataType, DataType),
+        (ProxyHScale, HScaleDataType, DataType),
+        (ProxyVScale, VScaleDataType, DataType),
+        (ProxyRadioButton, None, BoolOnlyDataType),
+        (ProxyTextView, TextViewDataType, DataType)
         ]:
         # Property overrides, used in the editor
         type_name = gobject.type_name(gobj)
 
         data_name = type_name + '::data-type'
         if editor:
-            prop_registry.override_simple(data_name, data_type, editor=editor)
+            prop_registry.override_simple(data_name, data_type, custom_editor=editor)
         else:
             prop_registry.override_simple(data_name, data_type)
 
         prop_registry.override_simple(type_name + '::model-attribute',
                                       ModelProperty)
 
-        if issubclass(gobj, RadioButton):
+        if issubclass(gobj, ProxyRadioButton):
             prop_registry.override_simple(type_name + '::data-value',
                                           DataValueProperty)
         # Register custom adapters, since gobject.new is broken in 2.6
@@ -341,8 +352,10 @@ def register_widgets():
         if gobj == ProxyComboBox:
             continue
 
-        klass = type('Kiwi%sAdapter', (PythonWidgetAdapter,),
-                     dict(object_type=gobj))
+        adapter_name = 'Kiwi%sAdapter' % gobj.__name__
+        klass = type(adapter_name, (PythonWidgetAdapter,),
+                     dict(object_type=gobj,
+                          __name__=adapter_name))
         adapter_registry.register_adapter(klass)
 
 if not environ.epydoc:
