@@ -28,8 +28,10 @@
 
 import datetime
 import gettext
+import pickle
 
 import gobject
+import pango
 import gtk
 from gtk import gdk
 
@@ -45,8 +47,6 @@ _ = lambda m: gettext.dgettext('kiwi', m)
 
 log = Logger('objectlist')
 
-str2type = converter.str_to_type
-
 def str2enum(value_name, enum_class):
     "converts a string to a enum"
     for _, enum in enum_class.__enum_values__.items():
@@ -113,9 +113,6 @@ class Column(PropertyObject, gobject.GObject):
       - B{radio}: bool I{False}
         -  If true render the column as a radio instead of toggle.
            Only applicable for columns with boolean data types.
-      - B{cache}: bool I{False}
-        -  If true, the value will only be fetched once, and the same value
-           will be reused for futher access.
       - B{use_stock}: bool I{False}
         - If true, this will be rendered as pixbuf from the value which
           should be a stock id.
@@ -127,6 +124,13 @@ class Column(PropertyObject, gobject.GObject):
         - If true, the text will be rendered with markup
       - B{expander}: bool I{False}
         - If True, this column will be used as the tree expander column
+      - B{ellipsize}: pango.EllipsizeMode I{pango.ELLIPSIZE_NONE}
+        - One of pango.ELLIPSIZE_{NONE, START, MIDDLE_END}, it describes
+          where characters should be removed in case ellipsization
+          (where to put the ...) is needed.
+      - B{font-desc}: str I{""}
+        - A string passed to pango.FontDescription, for instance "Sans" or
+          "Monospace 28".
     """
     __gtype_name__ = 'Column'
     gproperty('attribute', str, flags=(gobject.PARAM_READWRITE | gobject.PARAM_CONSTRUCT_ONLY))
@@ -144,12 +148,13 @@ class Column(PropertyObject, gobject.GObject):
     gproperty('editable', bool, default=False)
     gproperty('searchable', bool, default=False)
     gproperty('radio', bool, default=False)
-    gproperty('cache', bool, default=False)
     gproperty('use-stock', bool, default=False)
     gproperty('use-markup', bool, default=False)
     gproperty('icon-size', gtk.IconSize, default=gtk.ICON_SIZE_MENU)
     gproperty('editable-attribute', str)
     gproperty('expander', bool, False)
+    gproperty('ellipsize', pango.EllipsizeMode, default=pango.ELLIPSIZE_NONE)
+    gproperty('font-desc', str)
     #gproperty('title_pixmap', str)
 
     # This can be set in subclasses, to be able to allow custom
@@ -176,18 +181,18 @@ class Column(PropertyObject, gobject.GObject):
         @param data_type: the type of the attribute that will be inserted
             into the column.
 
-        @keyword title_pixmap: (TODO) if set to a filename a pixmap will be
+        @note: title_pixmap: (TODO) if set to a filename a pixmap will be
             used *instead* of the title set. The title string will still be
             used to identify the column in the column selection and in a
             tooltip, if a tooltip is not set.
         """
-
         # XXX: filter function?
         if ' ' in attribute:
             msg = ("The attribute can not contain spaces, otherwise I can"
                    " not find the value in the instances: %s" % attribute)
             raise AttributeError(msg)
 
+        self._objectlist = None
         self.compare = None
         self.from_string = None
 
@@ -225,9 +230,9 @@ class Column(PropertyObject, gobject.GObject):
         PropertyObject.__init__(self, **kwargs)
         gobject.GObject.__init__(self, attribute=attribute)
 
-    # This is meant to be subclassable, we're using kgetattr, as
-    # a staticmethod as an optimization, so we can avoid a function call.
-    get_attribute = staticmethod(kgetattr)
+    def __repr__(self):
+        namespace = self.__dict__.copy()
+        return "<%s: %s>" % (self.__class__.__name__, namespace)
 
     def prop_set_data_type(self, data):
         if data is not None:
@@ -236,14 +241,271 @@ class Column(PropertyObject, gobject.GObject):
             self.from_string = conv.from_string
         return data
 
-    def __repr__(self):
-        namespace = self.__dict__.copy()
-        return "<%s: %s>" % (self.__class__.__name__, namespace)
+    def attach(self, objectlist):
+        self._objectlist = objectlist
+
+        model = objectlist.get_model()
+        # You can't subclass bool, so this is okay
+        if (self.data_type is bool and self.format):
+            raise TypeError("format is not supported for boolean columns")
+
+        treeview_column = gtk.TreeViewColumn()
+
+        renderer, renderer_prop = self._guess_renderer_for_type(model)
+        if self.on_attach_renderer:
+            self.on_attach_renderer(renderer)
+        justify = self.justify
+        if justify == gtk.JUSTIFY_RIGHT:
+            xalign = 1.0
+        elif justify == gtk.JUSTIFY_CENTER:
+            xalign = 0.5
+        elif justify in (gtk.JUSTIFY_LEFT,
+                         gtk.JUSTIFY_FILL):
+            xalign = 0.0
+        else:
+            raise AssertionError
+        renderer.set_property("xalign", xalign)
+        treeview_column.set_property("alignment", xalign)
+
+        if self.ellipsize:
+            renderer.set_property('ellipsize', self.ellipsize)
+        if self.font_desc:
+            renderer.set_property('font-desc',
+                                  pango.FontDescription(self.font_desc))
+        if self.use_stock:
+            cell_data_func = self._cell_data_pixbuf_func
+        elif issubclass(self.data_type, enum):
+            cell_data_func = self._cell_data_combo_func
+        else:
+            cell_data_func = self._cell_data_text_func
+
+        if self.cell_data_func:
+            cell_data_func = self.cell_data_func
+
+        treeview_column.pack_start(renderer)
+        treeview_column.set_cell_data_func(renderer, cell_data_func,
+                                           (self, renderer_prop))
+        treeview_column.set_visible(self.visible)
+
+        if self.width:
+            treeview_column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+            treeview_column.set_fixed_width(self.width)
+
+        if self.expand:
+            # Default is False
+            treeview_column.set_expand(True)
+
+        if self.sorted:
+            treeview_column.set_sort_indicator(True)
+
+        if self.width:
+            self._autosize = False
+
+        if self.radio:
+            if not issubclass(self.data_type, bool):
+                raise TypeError("You can only use radio for boolean columns")
+
+        if self.expander:
+            self._treeview.set_expander_column(treeview_column)
+
+        # typelist here may be none. It's okay; justify_columns will try
+        # and use the specified justifications and if not present will
+        # not touch the column. When typelist is not set,
+        # append/add_list have a chance to fix up the remaining
+        # justification by looking at the first instance's data.
+#        self._justify_columns(columns, typelist)
+
+        self.treeview_column = treeview_column
+
+        return treeview_column
+
+    def _guess_renderer_for_type(self, model):
+        """Gusses which CellRenderer we should use for a given type.
+        It also set the property of the renderer that depends on the model,
+        in the renderer.
+        """
+
+        data_type = self.data_type
+        if data_type is bool:
+            renderer = gtk.CellRendererToggle()
+            if self.editable:
+                renderer.set_property('activatable', True)
+                # Boolean can be either a radio or a checkbox.
+                # Changes are handled by the toggled callback, which
+                # should only be connected if the column is editable.
+                if self.radio:
+                    renderer.set_radio(True)
+                    cb = self._on_renderer_toggle_radio__toggled
+                else:
+                    cb = self._on_renderer_toggle_check__toggled
+                renderer.connect('toggled', cb, model, self.attribute)
+            prop = 'active'
+        elif self.use_stock or data_type == gdk.Pixbuf:
+            renderer = gtk.CellRendererPixbuf()
+            prop = 'pixbuf'
+            if self.editable:
+                raise TypeError("use-stock columns cannot be editable")
+        elif issubclass(data_type, enum):
+            if data_type is enum:
+                raise TypeError("data_type must be a subclass of enum")
+
+            enum_model = gtk.ListStore(str, object)
+            items = data_type.names.items()
+            items.sort()
+            for key, value in items:
+                enum_model.append((key.lower().capitalize(), value))
+
+            renderer =  gtk.CellRendererCombo()
+            renderer.set_property('model', enum_model)
+            renderer.set_property('text-column', 0)
+            renderer.set_property('has-entry', True)
+            if self.editable:
+                renderer.set_property('editable', True)
+                renderer.connect('edited', self._on_renderer_combo__edited,
+                                 model, self.attribute, self)
+            prop = 'model'
+        elif issubclass(data_type, (datetime.date, datetime.time,
+                                    basestring, number,
+                                    currency)):
+            renderer = gtk.CellRendererText()
+            if self.use_markup:
+                prop = 'markup'
+            else:
+                prop = 'text'
+            if self.editable:
+                renderer.set_property('editable', True)
+                renderer.connect('edited', self._on_renderer_text__edited,
+                                 model, self.attribute, self,
+                                 self.from_string)
+
+        else:
+            raise ValueError("the type %s is not supported yet" % data_type)
+
+        return renderer, prop
+
+    # CellRenderers
+    def _cell_data_text_func(self, tree_column, renderer, model, treeiter,
+                             (column, renderer_prop)):
+        "To render the data of a cell renderer text"
+        row = model[treeiter]
+        if column.editable_attribute:
+            data = column.get_attribute(row[COL_MODEL],
+                                        column.editable_attribute, None)
+            data_type = column.data_type
+            if isinstance(renderer, gtk.CellRendererToggle):
+                renderer.set_property('activatable', data)
+            elif isinstance(renderer, gtk.CellRendererText):
+                renderer.set_property('editable', data)
+            else:
+                raise AssertionError
+
+        data = column.get_attribute(row[COL_MODEL],
+                                    column.attribute, None)
+
+        text = column.as_string(data)
+
+        renderer.set_property(renderer_prop, text)
+
+        if column.renderer_func:
+            column.renderer_func(renderer, data)
+
+    def _cell_data_pixbuf_func(self, tree_column, renderer, model, treeiter,
+                               (column, renderer_prop)):
+        "To render the data of a cell renderer pixbuf"
+        row = model[treeiter]
+        data = column.get_attribute(row[COL_MODEL],
+                                    column.attribute, None)
+        if data is not None:
+            pixbuf = self._objectlist.render_icon(data, column.icon_size)
+            renderer.set_property(renderer_prop, pixbuf)
+
+    def _cell_data_combo_func(self, tree_column, renderer, model, treeiter,
+                              (column, renderer_prop)):
+        row = model[treeiter]
+        data = column.get_attribute(row[COL_MODEL],
+                                    column.attribute, None)
+        text = column.as_string(data)
+        renderer.set_property('text', text.lower().capitalize())
+
+    def _on_renderer__toggled(self, renderer, path, column):
+        setattr(self._model[path][COL_MODEL], column.attribute,
+                not renderer.get_active())
+
+    def _on_renderer_toggle_check__toggled(self, renderer, path, model, attr):
+        obj = model[path][COL_MODEL]
+        value = not getattr(obj, attr, None)
+        setattr(obj, attr, value)
+        self._objectlist.emit('cell-edited', obj, attr)
+
+    def _on_renderer_toggle_radio__toggled(self, renderer, path, model, attr):
+        # Deactive old one
+        old = renderer.get_data('kiwilist::radio-active')
+
+        # If we don't have the radio-active set it means we're doing
+        # This for the first time, so scan and see which one is currently
+        # active, so we can deselect it
+        if not old:
+            # XXX: Handle multiple values set to True, this
+            #      algorithm just takes the first one it finds
+            for row in model:
+                obj = row[COL_MODEL]
+                value = getattr(obj, attr)
+                if value == True:
+                    old = obj
+                    break
+        if old:
+            setattr(old, attr, False)
+
+        # Active new and save a reference to the object of the
+        # previously selected row
+        new = model[path][COL_MODEL]
+        setattr(new, attr, True)
+        renderer.set_data('kiwilist::radio-active', new)
+        self._objectlist.emit('cell-edited', new, attr)
+
+    def _on_renderer_text__edited(self, renderer, path, text,
+                                  model, attr, column, from_string):
+        obj = model[path][COL_MODEL]
+        value = from_string(text)
+        setattr(obj, attr, value)
+        self._objectlist.emit('cell-edited', obj, attr)
+
+    def _on_renderer_combo__edited(self, renderer, path, text,
+                                   model, attr, column):
+        obj = model[path][COL_MODEL]
+        if not text:
+            return
+
+        value_model = renderer.get_property('model')
+        for row in value_model:
+            if row[0] == text:
+                value = row[1]
+                break
+        else:
+            raise AssertionError
+        setattr(obj, attr, value)
+        self._objectlist.emit('cell-edited', obj, attr)
+
+    def _on_renderer__edited(self, renderer, path, value, column):
+        data_type = column.data_type
+        if data_type in number:
+            value = data_type(value)
+
+        # XXX convert new_text to the proper data type
+        setattr(self._model[path][COL_MODEL], column.attribute, value)
+
+    # Public API
+
+    # This is meant to be subclassable, we're using kgetattr, as
+    # a staticmethod as an optimization, so we can avoid a function call.
+    get_attribute = staticmethod(kgetattr)
 
     def as_string(self, data):
         data_type = self.data_type
         if data is None:
             text = ''
+        elif self.format_func:
+            text = self.format_func(data)
         elif (self.format or
             data_type == float or
             data_type == Decimal or
@@ -254,13 +516,12 @@ class Column(PropertyObject, gobject.GObject):
             issubclass(data_type, enum)):
             conv = converter.get_converter(data_type)
             text = conv.as_string(data, format=self.format or None)
-        elif self.format_func:
-            text = self.format_func(data)
         else:
             text = data
 
         return text
 
+
 class SequentialColumn(Column):
     """I am a column which will display a sequence of numbers, which
     represent the row number. The value is independent of the data in
@@ -505,6 +766,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
                  sortable=False,
                  model=None):
         """
+        Create a new ObjectList object.
         @param columns:       a list of L{Column}s
         @param objects:       a list of objects to be inserted or None
         @param mode:          selection mode
@@ -517,10 +779,12 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         if isinstance(columns, Column):
             columns = [columns]
         elif not isinstance(columns, list):
-            raise TypeError("columns must be a list or a Column")
+            raise TypeError(
+                "columns must be a list or a Column, not %r" % (columns,))
 
         if not isinstance(mode, gtk.SelectionMode):
-            raise TypeError("mode must be an gtk.SelectionMode enum")
+            raise TypeError(
+                "mode must be an gtk.SelectionMode enum, not %r" % (mode,))
         # gtk.SELECTION_EXTENDED & gtk.SELECTION_MULTIPLE are both 3.
         # so we can't do this check.
         #elif mode == gtk.SELECTION_EXTENDED:
@@ -531,7 +795,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         self._columns = []
         # Mapping of instance id -> treeiter
         self._iters = {}
-        self._cell_data_caches = {}
         self._autosize = True
         self._vscrollbar = None
 
@@ -847,106 +1110,14 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         self._sortable = self._sortable or sorted
 
         for column in columns:
-            self._setup_column(column)
+            self._attach_column(column)
 
         if not expand:
             column = gtk.TreeViewColumn()
             self._treeview.append_column(column)
 
-    def _setup_column(self, column):
-        # You can't subclass bool, so this is okay
-        if (column.data_type is bool and column.format):
-            raise TypeError("format is not supported for boolean columns")
-
-        index = self._columns.index(column)
-        treeview_column = self._treeview.get_column(index)
-        if treeview_column is None:
-            treeview_column = self._create_column(column)
-
-        if self._sortable:
-            self._model.set_sort_func(index,
-                                      self._model_sort_func,
-                                      (column, column.attribute))
-            treeview_column.set_sort_column_id(index)
-
-        if column.sorted:
-            self._model.set_sort_column_id(index, column.order)
-
-        renderer, renderer_prop = self._guess_renderer_for_type(column)
-        if column.on_attach_renderer:
-            column.on_attach_renderer(renderer)
-        justify = column.justify
-        if justify == gtk.JUSTIFY_RIGHT:
-            xalign = 1.0
-        elif justify == gtk.JUSTIFY_CENTER:
-            xalign = 0.5
-        elif justify in (gtk.JUSTIFY_LEFT,
-                         gtk.JUSTIFY_FILL):
-            xalign = 0.0
-        else:
-            raise AssertionError
-        renderer.set_property("xalign", xalign)
-        treeview_column.set_property("alignment", xalign)
-
-        if column.use_stock:
-            cell_data_func = self._cell_data_pixbuf_func
-        elif issubclass(column.data_type, enum):
-            cell_data_func = self._cell_data_combo_func
-        else:
-            cell_data_func = self._cell_data_text_func
-
-        if column.cell_data_func:
-            cell_data_func = column.cell_data_func
-        elif column.cache:
-            self._cell_data_caches[column.attribute] = {}
-
-        treeview_column.pack_start(renderer)
-        treeview_column.set_cell_data_func(renderer, cell_data_func,
-                                           (column, renderer_prop))
-        treeview_column.set_visible(column.visible)
-
-        if column.width:
-            treeview_column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-            treeview_column.set_fixed_width(column.width)
-        if column.tooltip:
-            widget = self._get_column_button(treeview_column)
-            if widget is not None:
-                self._tooltips.set_tip(widget, column.tooltip)
-
-        if column.expand:
-            # Default is False
-            treeview_column.set_expand(True)
-
-        if column.sorted:
-            treeview_column.set_sort_indicator(True)
-
-        if column.width:
-            self._autosize = False
-
-        if column.searchable:
-            if not issubclass(column.data_type, basestring):
-                raise TypeError("Unsupported data type for "
-                                "searchable column: %s" % column.data_type)
-            self._treeview.set_search_column(index)
-            self._treeview.set_search_equal_func(self._treeview_search_equal_func,
-                                                 column)
-
-        if column.radio:
-            if not issubclass(column.data_type, bool):
-                raise TypeError("You can only use radio for boolean columns")
-
-        if column.expander:
-            self._treeview.set_expander_column(treeview_column)
-
-        # typelist here may be none. It's okay; justify_columns will try
-        # and use the specified justifications and if not present will
-        # not touch the column. When typelist is not set,
-        # append/add_list have a chance to fix up the remaining
-        # justification by looking at the first instance's data.
-#        self._justify_columns(columns, typelist)
-
-    def _create_column(self, column):
-        treeview_column = gtk.TreeViewColumn()
+    def _attach_column(self, column):
+        treeview_column = column.attach(self)
         # we need to set our own widget because otherwise
         # __get_column_button won't work
 
@@ -962,72 +1133,32 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         button = self._get_column_button(treeview_column)
         button.connect('button-release-event',
                        self._on_treeview_header__button_release_event)
-        return treeview_column
 
-    def _guess_renderer_for_type(self, column):
-        """Gusses which CellRenderer we should use for a given type.
-        It also set the property of the renderer that depends on the model,
-        in the renderer.
-        """
+        index = self._columns.index(column)
+        if self._sortable:
+            self._model.set_sort_func(index,
+                                      self._model_sort_func,
+                                      (column, column.attribute))
+            treeview_column.set_sort_column_id(index)
 
-        # TODO: Move to column
-        data_type = column.data_type
-        if data_type is bool:
-            renderer = gtk.CellRendererToggle()
-            if column.editable:
-                renderer.set_property('activatable', True)
-                # Boolean can be either a radio or a checkbox.
-                # Changes are handled by the toggled callback, which
-                # should only be connected if the column is editable.
-                if column.radio:
-                    renderer.set_radio(True)
-                    cb = self._on_renderer_toggle_radio__toggled
-                else:
-                    cb = self._on_renderer_toggle_check__toggled
-                renderer.connect('toggled', cb, self._model, column.attribute)
-            prop = 'active'
-        elif column.use_stock or data_type == gdk.Pixbuf:
-            renderer = gtk.CellRendererPixbuf()
-            prop = 'pixbuf'
-            if column.editable:
-                raise TypeError("use-stock columns cannot be editable")
-        elif issubclass(data_type, enum):
-            if data_type is enum:
-                raise TypeError("data_type must be a subclass of enum")
+        if column.sorted:
+            self._model.set_sort_column_id(index, column.order)
 
-            model = gtk.ListStore(str, object)
-            items = data_type.names.items()
-            items.sort()
-            for key, value in items:
-                model.append((key.lower().capitalize(), value))
+        if column.searchable:
+            if not issubclass(column.data_type, basestring):
+                raise TypeError("Unsupported data type for "
+                                "searchable column: %s" % column.data_type)
+            self._treeview.set_search_column(index)
+            self._treeview.set_search_equal_func(
+                self._treeview_search_equal_func, column)
 
-            renderer =  gtk.CellRendererCombo()
-            renderer.set_property('model', model)
-            renderer.set_property('text-column', 0)
-            renderer.set_property('has-entry', True)
-            if column.editable:
-                renderer.set_property('editable', True)
-                renderer.connect('edited', self._on_renderer_combo__edited,
-                                 self._model, column.attribute, column)
-            prop = 'model'
-        elif issubclass(data_type, (datetime.date, datetime.time,
-                                    basestring, number,
-                                    currency)):
-            renderer = gtk.CellRendererText()
-            if column.use_markup:
-                prop = 'markup'
-            else:
-                prop = 'text'
-            if column.editable:
-                renderer.set_property('editable', True)
-                renderer.connect('edited', self._on_renderer_text__edited,
-                                 self._model, column.attribute, column,
-                                 column.from_string)
+        if column.expander:
+            self._treeview.set_expander_column(treeview_column)
 
-        else:
-            raise ValueError("the type %s is not supported yet" % data_type)
-
-        return renderer, prop
+        if column.tooltip:
+            widget = self._get_column_button(treeview_column)
+            if widget is not None:
+                self._tooltips.set_tip(widget, column.tooltip)
 
     # selection methods
     def _select_and_focus_row(self, row_iter):
@@ -1116,6 +1247,8 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         mode = selection.get_mode()
         if mode == gtk.SELECTION_MULTIPLE:
             item = self.get_selected_rows()
+        elif mode == gtk.SELECTION_NONE:
+            return
         else:
             item = self.get_selected()
         return item
@@ -1144,126 +1277,11 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
             if item:
                 self.emit('double-click', item)
 
-    # CellRenderers
-    def _cell_data_text_func(self, tree_column, renderer, model, treeiter,
-                             (column, renderer_prop)):
-        "To render the data of a cell renderer text"
-        row = model[treeiter]
-        if column.editable_attribute:
-            data = column.get_attribute(row[COL_MODEL],
-                                        column.editable_attribute, None)
-            data_type = column.data_type
-            if isinstance(renderer, gtk.CellRendererToggle):
-                renderer.set_property('activatable', data)
-            elif isinstance(renderer, gtk.CellRendererText):
-                renderer.set_property('editable', data)
-            else:
-                raise AssertionError
 
-        if column.cache:
-            cache = self._cell_data_caches[column.attribute]
-            path = row.path[0]
-            if path in cache:
-                data = cache[path]
-            else:
-                data = column.get_attribute(row[COL_MODEL],
-                                            column.attribute, None)
-                cache[path] = data
-        else:
-            data = column.get_attribute(row[COL_MODEL],
-                                        column.attribute, None)
-
-        text = column.as_string(data)
-
-        renderer.set_property(renderer_prop, text)
-
-        if column.renderer_func:
-            column.renderer_func(renderer, data)
-
-    def _cell_data_pixbuf_func(self, tree_column, renderer, model, treeiter,
-                               (column, renderer_prop)):
-        "To render the data of a cell renderer pixbuf"
-        row = model[treeiter]
-        data = column.get_attribute(row[COL_MODEL],
-                                    column.attribute, None)
-        if data is not None:
-            pixbuf = self.render_icon(data, column.icon_size)
-            renderer.set_property(renderer_prop, pixbuf)
-
-    def _cell_data_combo_func(self, tree_column, renderer, model, treeiter,
-                              (column, renderer_prop)):
-        row = model[treeiter]
-        data = column.get_attribute(row[COL_MODEL],
-                                    column.attribute, None)
-        text = column.as_string(data)
-        renderer.set_property('text', text.lower().capitalize())
-
-    def _on_renderer__toggled(self, renderer, path, column):
-        setattr(self._model[path][COL_MODEL], column.attribute,
-                not renderer.get_active())
-
-    def _on_renderer_toggle_check__toggled(self, renderer, path, model, attr):
-        obj = model[path][COL_MODEL]
-        value = not getattr(obj, attr, None)
-        setattr(obj, attr, value)
-        self.emit('cell-edited', obj, attr)
-
-    def _on_renderer_toggle_radio__toggled(self, renderer, path, model, attr):
-        # Deactive old one
-        old = renderer.get_data('kiwilist::radio-active')
-
-        # If we don't have the radio-active set it means we're doing
-        # This for the first time, so scan and see which one is currently
-        # active, so we can deselect it
-        if not old:
-            # XXX: Handle multiple values set to True, this
-            #      algorithm just takes the first one it finds
-            for row in self._model:
-                obj = row[COL_MODEL]
-                value = getattr(obj, attr)
-                if value == True:
-                    old = obj
-                    break
-        if old:
-            setattr(old, attr, False)
-
-        # Active new and save a reference to the object of the
-        # previously selected row
-        new = model[path][COL_MODEL]
-        setattr(new, attr, True)
-        renderer.set_data('kiwilist::radio-active', new)
-        self.emit('cell-edited', new, attr)
-
-    def _on_renderer_text__edited(self, renderer, path, text,
-                                  model, attr, column, from_string):
-        obj = model[path][COL_MODEL]
-        value = from_string(text)
-        setattr(obj, attr, value)
-        self.emit('cell-edited', obj, attr)
-
-    def _on_renderer_combo__edited(self, renderer, path, text,
-                                  model, attr, column):
-        obj = model[path][COL_MODEL]
-        if not text:
-            return
-
-        value_model = renderer.get_property('model')
-        for row in value_model:
-            if row[0] == text:
-                value = row[1]
-                break
-        else:
-            raise AssertionError
-        setattr(obj, attr, value)
-        self.emit('cell-edited', obj, attr)
-
-    def _on_renderer__edited(self, renderer, path, value, column):
-        data_type = column.data_type
-        if data_type in number:
-            value = data_type(value)
-
-        # XXX convert new_text to the proper data type
-        setattr(self._model[path][COL_MODEL], column.attribute, value)
+    def _on_treeview__source_drag_data_get(self, treeview, context,
+                                           selection, info, timestamp):
+        item = self.get_selected()
+        selection.set('OBJECTLIST_ROW', 8, pickle.dumps(item))
 
     # hacks
     def _get_column_button(self, column):
@@ -1339,6 +1357,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     def get_treeview_column(self, column):
         """
+        Get the treeview column given an objectlist column
         @param column: a @Column
         """
         if not isinstance(column, Column):
@@ -1347,9 +1366,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         if not column in self._columns:
             raise ValueError
 
-        index = self._columns.index(column)
-        tree_columns = self._treeview.get_columns()
-        return tree_columns[index]
+        return column.treeview_column
 
     def grab_focus(self):
         """
@@ -1374,6 +1391,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     def set_columns(self, columns):
         """
+        Set columns.
         @param columns: a sequence of L{Column} objects.
         """
 
@@ -1409,21 +1427,11 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         if not treeiter:
             return False
 
-        # Remove any references to this path
-        self._clear_cache_for_iter(treeiter)
-
         # All references to the iter gone, now it can be removed
         self._model.remove(treeiter)
 
         return True
 
-    def _clear_cache_for_iter(self, treeiter):
-        # Not as inefficent as it looks
-        path = self._model[treeiter].path[0]
-        for cache in self._cell_data_caches.values():
-            if path in cache:
-                del cache[path]
-
     def remove(self, instance, select=False):
         """Remove an instance from the list.
         If the instance is not in the list it returns False. Otherwise it
@@ -1453,7 +1461,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         if not objid in self._iters:
             raise ValueError("instance %r is not in the list" % instance)
         treeiter = self._iters[objid]
-        self._clear_cache_for_iter(treeiter)
         self._model.row_changed(self._model[treeiter].path, treeiter)
 
     def refresh(self, view_only=False):
@@ -1584,11 +1591,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         self._model.clear()
         self._iters = {}
 
-        # Don't clear the whole cache, just the
-        # individual column caches
-        for key in self._cell_data_caches:
-            self._cell_data_caches[key] = {}
-
     def get_next(self, instance):
         """
         Returns the item after instance in the list.
@@ -1636,7 +1638,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     def get_selected_row_number(self):
         """
-        @returns: the selected row number or None if no rows were selected
+        Get the selected row number or None if no rows were selected
         """
         selection = self._treeview.get_selection()
         if selection.get_mode() == gtk.SELECTION_MULTIPLE:
@@ -1663,6 +1665,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     def set_headers_visible(self, value):
         """
+        Show or hide the headers.
         @param value: if true, shows the headers, if false hide then
         """
         self._treeview.set_headers_visible(value)
@@ -1688,8 +1691,27 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         focus_padding = treeview.style_get_property('focus-line-width') * 2
         treeview.set_size_request(-1, header_h + (rows * (h + focus_padding)))
 
+    def enable_dnd(self):
+        """
+        Enables Drag and Drop from this object list
+        """
+        self._treeview.connect('drag-data-get',
+                               self._on_treeview__source_drag_data_get)
+        self._treeview.drag_source_set(
+            gtk.gdk.BUTTON1_MASK, self.get_dnd_targets(),
+            gtk.gdk.ACTION_LINK)
+
+    def get_dnd_targets(self):
+        """
+        Get a list of dnd targets ObjectList supports
+        """
+        return [
+            ('OBJECTLIST_ROW', 0, 10),
+            ]
+
 type_register(ObjectList)
 
+
 class ObjectTree(ObjectList):
     """
     Signals
@@ -1740,6 +1762,7 @@ class ObjectTree(ObjectList):
 
     def append(self, parent, instance, select=False):
         """
+        Append the selected row in an instance.
         @param parent: Object or None, representing the parent
         @param instance: the instance to be added
         @param select: select the row
@@ -1749,6 +1772,7 @@ class ObjectTree(ObjectList):
 
     def prepend(self, parent, instance, select=False):
         """
+        Prepend the selected row in an instance.
         @param parent: Object or None, representing the parent
         @param instance: the instance to be added
         @param select: select the row
@@ -1796,8 +1820,10 @@ class ListLabel(gtk.HBox):
     to vertically align a label with a column
     """
 
-    def __init__(self, klist, column, label='', value_format='%s'):
+    def __init__(self, klist, column, label='', value_format='%s',
+                 font_desc=None):
         """
+        Constructor.
         @param klist:        list to follow
         @type klist:         kiwi.ui.objectlist.ObjectList
         @param column:       name of a column in a klist
@@ -1823,6 +1849,10 @@ class ListLabel(gtk.HBox):
 
         self._create_ui()
 
+        if font_desc:
+            self._value_widget.modify_font(font_desc)
+            self._label_widget.modify_font(font_desc)
+
     # Public API
 
     def set_value(self, value):
@@ -1905,8 +1935,9 @@ class SummaryLabel(ListLabel):
     Please note that I only know how to handle number column
     data types and I will complain if you give me something else."""
 
-    def __init__(self, klist, column, label=_('Total:'), value_format='%s'):
-        ListLabel.__init__(self, klist, column, label, value_format)
+    def __init__(self, klist, column, label=_('Total:'), value_format='%s',
+                 font_desc=None):
+        ListLabel.__init__(self, klist, column, label, value_format, font_desc)
         if not issubclass(self._column.data_type, number):
             raise TypeError("data_type of column must be a number, not %r",
                             self._column.data_type)
@@ -1921,7 +1952,7 @@ class SummaryLabel(ListLabel):
         attr = column.attribute
         get_attribute = column.get_attribute
 
-        value = sum([get_attribute(obj, attr, 0) for obj in self._klist],
+        value = sum([get_attribute(obj, attr, 0) or 0 for obj in self._klist],
                     column.data_type('0'))
 
         self.set_value(column.as_string(value))
